# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/galacteek.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_GalacteekWindow(object):
    def setupUi(self, GalacteekWindow):
        GalacteekWindow.setObjectName("GalacteekWindow")
        GalacteekWindow.resize(697, 446)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(GalacteekWindow.sizePolicy().hasHeightForWidth())
        GalacteekWindow.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/share/icons/ipfs-logo-128-ice-text.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        GalacteekWindow.setWindowIcon(icon)
        self.centralwidget = QtWidgets.QWidget(GalacteekWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSizeConstraint(QtWidgets.QLayout.SetMaximumSize)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.openBrowserTabButton = QtWidgets.QPushButton(self.centralwidget)
        self.openBrowserTabButton.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/share/icons/ipfs-logo-128-ice.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.openBrowserTabButton.setIcon(icon1)
        self.openBrowserTabButton.setIconSize(QtCore.QSize(24, 24))
        self.openBrowserTabButton.setObjectName("openBrowserTabButton")
        self.horizontalLayout_2.addWidget(self.openBrowserTabButton)
        self.myFilesButton = QtWidgets.QToolButton(self.centralwidget)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/share/icons/folder-open.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.myFilesButton.setIcon(icon2)
        self.myFilesButton.setIconSize(QtCore.QSize(24, 24))
        self.myFilesButton.setObjectName("myFilesButton")
        self.horizontalLayout_2.addWidget(self.myFilesButton)
        self.mediaPlayerButton = QtWidgets.QToolButton(self.centralwidget)
        self.mediaPlayerButton.setText("")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/share/icons/multimedia.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.mediaPlayerButton.setIcon(icon3)
        self.mediaPlayerButton.setIconSize(QtCore.QSize(24, 24))
        self.mediaPlayerButton.setObjectName("mediaPlayerButton")
        self.horizontalLayout_2.addWidget(self.mediaPlayerButton)
        spacerItem = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.manageKeysButton = QtWidgets.QToolButton(self.centralwidget)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/share/icons/lock-and-key.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.manageKeysButton.setIcon(icon4)
        self.manageKeysButton.setIconSize(QtCore.QSize(24, 24))
        self.manageKeysButton.setObjectName("manageKeysButton")
        self.horizontalLayout_2.addWidget(self.manageKeysButton, 0, QtCore.Qt.AlignLeft)
        self.hashmarksButton = QtWidgets.QPushButton(self.centralwidget)
        self.hashmarksButton.setText("")
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/share/icons/hashmarks.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.hashmarksButton.setIcon(icon5)
        self.hashmarksButton.setIconSize(QtCore.QSize(24, 24))
        self.hashmarksButton.setObjectName("hashmarksButton")
        self.horizontalLayout_2.addWidget(self.hashmarksButton)
        spacerItem1 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.writeNewDocumentButton = QtWidgets.QPushButton(self.centralwidget)
        self.writeNewDocumentButton.setText("")
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/share/icons/write-document.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.writeNewDocumentButton.setIcon(icon6)
        self.writeNewDocumentButton.setIconSize(QtCore.QSize(24, 24))
        self.writeNewDocumentButton.setObjectName("writeNewDocumentButton")
        self.horizontalLayout_2.addWidget(self.writeNewDocumentButton)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem2)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setSpacing(4)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.hLayoutProfile = QtWidgets.QHBoxLayout()
        self.hLayoutProfile.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.hLayoutProfile.setSpacing(4)
        self.hLayoutProfile.setObjectName("hLayoutProfile")
        self.peersMgrButton = QtWidgets.QToolButton(self.centralwidget)
        self.peersMgrButton.setText("")
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/share/icons/peers.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.peersMgrButton.setIcon(icon7)
        self.peersMgrButton.setIconSize(QtCore.QSize(24, 24))
        self.peersMgrButton.setObjectName("peersMgrButton")
        self.hLayoutProfile.addWidget(self.peersMgrButton)
        self.profileEditButton = QtWidgets.QToolButton(self.centralwidget)
        self.profileEditButton.setText("")
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/share/icons/profile-user.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.profileEditButton.setIcon(icon8)
        self.profileEditButton.setIconSize(QtCore.QSize(24, 24))
        self.profileEditButton.setObjectName("profileEditButton")
        self.hLayoutProfile.addWidget(self.profileEditButton)
        self.horizontalLayout_4.addLayout(self.hLayoutProfile)
        self.label = QtWidgets.QLabel(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setTextFormat(QtCore.Qt.PlainText)
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setWordWrap(False)
        self.label.setIndent(0)
        self.label.setTextInteractionFlags(QtCore.Qt.NoTextInteraction)
        self.label.setObjectName("label")
        self.horizontalLayout_4.addWidget(self.label)
        self.ipfsSearch = QtWidgets.QLineEdit(self.centralwidget)
        self.ipfsSearch.setMaximumSize(QtCore.QSize(230, 16777215))
        self.ipfsSearch.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.ipfsSearch.setMaxLength(128)
        self.ipfsSearch.setObjectName("ipfsSearch")
        self.horizontalLayout_4.addWidget(self.ipfsSearch)
        self.ipfsSearchButton = QtWidgets.QToolButton(self.centralwidget)
        self.ipfsSearchButton.setText("")
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/share/icons/ipfs-search.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.ipfsSearchButton.setIcon(icon9)
        self.ipfsSearchButton.setIconSize(QtCore.QSize(24, 24))
        self.ipfsSearchButton.setObjectName("ipfsSearchButton")
        self.horizontalLayout_4.addWidget(self.ipfsSearchButton)
        self.pinAllGlobalButton = QtWidgets.QToolButton(self.centralwidget)
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(":/share/icons/pin-black.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pinAllGlobalButton.setIcon(icon10)
        self.pinAllGlobalButton.setIconSize(QtCore.QSize(24, 24))
        self.pinAllGlobalButton.setObjectName("pinAllGlobalButton")
        self.horizontalLayout_4.addWidget(self.pinAllGlobalButton)
        self.clipboardMultiLoader = QtWidgets.QToolButton(self.centralwidget)
        self.clipboardMultiLoader.setText("")
        icon11 = QtGui.QIcon()
        icon11.addPixmap(QtGui.QPixmap(":/share/icons/clipboard.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.clipboardMultiLoader.setIcon(icon11)
        self.clipboardMultiLoader.setIconSize(QtCore.QSize(24, 24))
        self.clipboardMultiLoader.setObjectName("clipboardMultiLoader")
        self.horizontalLayout_4.addWidget(self.clipboardMultiLoader)
        self.horizontalLayout_2.addLayout(self.horizontalLayout_4)
        self.horizontalLayout.addLayout(self.horizontalLayout_2)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget.setObjectName("tabWidget")
        self.tabBlank = QtWidgets.QWidget()
        self.tabBlank.setObjectName("tabBlank")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.tabBlank)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.tabWidget.addTab(self.tabBlank, "")
        self.verticalLayout.addWidget(self.tabWidget)
        self.verticalLayout_3.addLayout(self.verticalLayout)
        GalacteekWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(GalacteekWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 697, 20))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuEdit = QtWidgets.QMenu(self.menubar)
        self.menuEdit.setObjectName("menuEdit")
        self.menuUser_Profile = QtWidgets.QMenu(self.menuEdit)
        self.menuUser_Profile.setObjectName("menuUser_Profile")
        self.menuAbout = QtWidgets.QMenu(self.menubar)
        self.menuAbout.setObjectName("menuAbout")
        self.menuTools = QtWidgets.QMenu(self.menubar)
        self.menuTools.setObjectName("menuTools")
        GalacteekWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(GalacteekWindow)
        self.statusbar.setObjectName("statusbar")
        GalacteekWindow.setStatusBar(self.statusbar)
        self.actionQuit = QtWidgets.QAction(GalacteekWindow)
        self.actionQuit.setPriority(QtWidgets.QAction.LowPriority)
        self.actionQuit.setObjectName("actionQuit")
        self.actionAddProfile = QtWidgets.QAction(GalacteekWindow)
        self.actionAddProfile.setObjectName("actionAddProfile")
        self.actionShowPeersInformation = QtWidgets.QAction(GalacteekWindow)
        self.actionShowPeersInformation.setObjectName("actionShowPeersInformation")
        self.actionShowIpfsRefs = QtWidgets.QAction(GalacteekWindow)
        self.actionShowIpfsRefs.setObjectName("actionShowIpfsRefs")
        self.actionSettings = QtWidgets.QAction(GalacteekWindow)
        self.actionSettings.setObjectName("actionSettings")
        self.actionAboutGalacteek = QtWidgets.QAction(GalacteekWindow)
        self.actionAboutGalacteek.setObjectName("actionAboutGalacteek")
        self.actionCloseAllTabs = QtWidgets.QAction(GalacteekWindow)
        self.actionCloseAllTabs.setPriority(QtWidgets.QAction.HighPriority)
        self.actionCloseAllTabs.setObjectName("actionCloseAllTabs")
        self.actionNew_Profile = QtWidgets.QAction(GalacteekWindow)
        self.actionNew_Profile.setObjectName("actionNew_Profile")
        self.actionDonate = QtWidgets.QAction(GalacteekWindow)
        self.actionDonate.setObjectName("actionDonate")
        self.actionEvent_log = QtWidgets.QAction(GalacteekWindow)
        self.actionEvent_log.setObjectName("actionEvent_log")
        self.menuFile.addAction(self.actionQuit)
        self.menuFile.addAction(self.actionCloseAllTabs)
        self.menuUser_Profile.addAction(self.actionNew_Profile)
        self.menuEdit.addAction(self.actionSettings)
        self.menuEdit.addAction(self.menuUser_Profile.menuAction())
        self.menuAbout.addAction(self.actionAboutGalacteek)
        self.menuAbout.addAction(self.actionDonate)
        self.menuTools.addAction(self.actionEvent_log)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuEdit.menuAction())
        self.menubar.addAction(self.menuTools.menuAction())
        self.menubar.addAction(self.menuAbout.menuAction())

        self.retranslateUi(GalacteekWindow)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(GalacteekWindow)

    def retranslateUi(self, GalacteekWindow):
        _translate = QtCore.QCoreApplication.translate
        GalacteekWindow.setWindowTitle(_translate("GalacteekWindow", "Galacteek"))
        self.openBrowserTabButton.setToolTip(_translate("GalacteekWindow", "<html><head/><body><p>Browse</p></body></html>"))
        self.myFilesButton.setToolTip(_translate("GalacteekWindow", "<html><head/><body><p>File Manager</p></body></html>"))
        self.myFilesButton.setText(_translate("GalacteekWindow", "..."))
        self.mediaPlayerButton.setToolTip(_translate("GalacteekWindow", "<html><head/><body><p>Media Player</p></body></html>"))
        self.manageKeysButton.setToolTip(_translate("GalacteekWindow", "<html><head/><body><p>IPFS keys</p></body></html>"))
        self.manageKeysButton.setText(_translate("GalacteekWindow", "..."))
        self.hashmarksButton.setToolTip(_translate("GalacteekWindow", "<html><head/><body><p>Hashmarks</p></body></html>"))
        self.writeNewDocumentButton.setToolTip(_translate("GalacteekWindow", "<html><head/><body><p>Write and import text document</p></body></html>"))
        self.peersMgrButton.setToolTip(_translate("GalacteekWindow", "<html><head/><body><p>Peers</p></body></html>"))
        self.profileEditButton.setToolTip(_translate("GalacteekWindow", "<html><head/><body><p>User Profile</p></body></html>"))
        self.label.setText(_translate("GalacteekWindow", "Search"))
        self.ipfsSearchButton.setToolTip(_translate("GalacteekWindow", "<html><head/><body><p>Search content on IPFS (ipfs-search.com)</p></body></html>"))
        self.pinAllGlobalButton.setToolTip(_translate("GalacteekWindow", "<html><head/><body><p>Global automatic pinning (enables automatic pinning for all browsing sessions)</p></body></html>"))
        self.pinAllGlobalButton.setText(_translate("GalacteekWindow", "..."))
        self.menuFile.setTitle(_translate("GalacteekWindow", "File"))
        self.menuEdit.setTitle(_translate("GalacteekWindow", "Edit"))
        self.menuUser_Profile.setTitle(_translate("GalacteekWindow", "User Profile"))
        self.menuAbout.setTitle(_translate("GalacteekWindow", "Help"))
        self.menuTools.setTitle(_translate("GalacteekWindow", "Tools"))
        self.actionQuit.setText(_translate("GalacteekWindow", "Quit"))
        self.actionQuit.setShortcut(_translate("GalacteekWindow", "Ctrl+Q"))
        self.actionAddProfile.setText(_translate("GalacteekWindow", "Add Profile"))
        self.actionShowPeersInformation.setText(_translate("GalacteekWindow", "Show peers information"))
        self.actionShowIpfsRefs.setText(_translate("GalacteekWindow", "Show refs"))
        self.actionSettings.setText(_translate("GalacteekWindow", "Settings"))
        self.actionAboutGalacteek.setText(_translate("GalacteekWindow", "About"))
        self.actionCloseAllTabs.setText(_translate("GalacteekWindow", "Close all tabs"))
        self.actionNew_Profile.setText(_translate("GalacteekWindow", "New Profile"))
        self.actionDonate.setText(_translate("GalacteekWindow", "Donate"))
        self.actionEvent_log.setText(_translate("GalacteekWindow", "Event log"))

from . import galacteek_rc
