# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/addhashmarkdialog.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_AddHashmarkDialog(object):
    def setupUi(self, AddHashmarkDialog):
        AddHashmarkDialog.setObjectName("AddHashmarkDialog")
        AddHashmarkDialog.resize(1071, 390)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(AddHashmarkDialog.sizePolicy().hasHeightForWidth())
        AddHashmarkDialog.setSizePolicy(sizePolicy)
        AddHashmarkDialog.setMaximumSize(QtCore.QSize(1071, 16777215))
        self.formLayout_2 = QtWidgets.QFormLayout(AddHashmarkDialog)
        self.formLayout_2.setObjectName("formLayout_2")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.resourceLabel = QtWidgets.QLabel(AddHashmarkDialog)
        self.resourceLabel.setMinimumSize(QtCore.QSize(800, 0))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.resourceLabel.setFont(font)
        self.resourceLabel.setText("")
        self.resourceLabel.setObjectName("resourceLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.resourceLabel)
        self.label = QtWidgets.QLabel(AddHashmarkDialog)
        self.label.setObjectName("label")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label)
        self.title = QtWidgets.QLineEdit(AddHashmarkDialog)
        self.title.setObjectName("title")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.title)
        self.label_2 = QtWidgets.QLabel(AddHashmarkDialog)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.category = QtWidgets.QComboBox(AddHashmarkDialog)
        self.category.setObjectName("category")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.category)
        self.label_4 = QtWidgets.QLabel(AddHashmarkDialog)
        self.label_4.setObjectName("label_4")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.label_4)
        self.newCategory = QtWidgets.QLineEdit(AddHashmarkDialog)
        self.newCategory.setMaxLength(128)
        self.newCategory.setObjectName("newCategory")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.newCategory)
        self.label_3 = QtWidgets.QLabel(AddHashmarkDialog)
        self.label_3.setText("")
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.LabelRole, self.label_3)
        self.share = QtWidgets.QCheckBox(AddHashmarkDialog)
        self.share.setObjectName("share")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.FieldRole, self.share)
        self.label_5 = QtWidgets.QLabel(AddHashmarkDialog)
        self.label_5.setObjectName("label_5")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.label_5)
        self.tags = QtWidgets.QLineEdit(AddHashmarkDialog)
        self.tags.setMaxLength(256)
        self.tags.setObjectName("tags")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.tags)
        self.label_6 = QtWidgets.QLabel(AddHashmarkDialog)
        self.label_6.setObjectName("label_6")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.label_6)
        self.comment = QtWidgets.QLineEdit(AddHashmarkDialog)
        self.comment.setObjectName("comment")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.FieldRole, self.comment)
        self.label_7 = QtWidgets.QLabel(AddHashmarkDialog)
        self.label_7.setObjectName("label_7")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_7)
        self.description = QtWidgets.QTextEdit(AddHashmarkDialog)
        self.description.setAcceptRichText(False)
        self.description.setObjectName("description")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.description)
        self.formLayout_2.setLayout(2, QtWidgets.QFormLayout.FieldRole, self.formLayout)
        self.buttonBox = QtWidgets.QDialogButtonBox(AddHashmarkDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.formLayout_2.setWidget(14, QtWidgets.QFormLayout.SpanningRole, self.buttonBox)

        self.retranslateUi(AddHashmarkDialog)
        self.buttonBox.accepted.connect(AddHashmarkDialog.accept)
        self.buttonBox.rejected.connect(AddHashmarkDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(AddHashmarkDialog)

    def retranslateUi(self, AddHashmarkDialog):
        _translate = QtCore.QCoreApplication.translate
        AddHashmarkDialog.setWindowTitle(_translate("AddHashmarkDialog", "Add hashmark dialog"))
        self.label.setText(_translate("AddHashmarkDialog", "Use title for hashmark"))
        self.label_2.setText(_translate("AddHashmarkDialog", "Category"))
        self.label_4.setText(_translate("AddHashmarkDialog", "or create new category, e.g news/world"))
        self.share.setText(_translate("AddHashmarkDialog", "Share (public)"))
        self.label_5.setText(_translate("AddHashmarkDialog", "Tags list"))
        self.label_6.setText(_translate("AddHashmarkDialog", "Comment"))
        self.label_7.setText(_translate("AddHashmarkDialog", "Description"))

