# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/settings.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_SettingsDialogForm(object):
    def setupUi(self, SettingsDialogForm):
        SettingsDialogForm.setObjectName("SettingsDialogForm")
        SettingsDialogForm.resize(724, 478)
        self.verticalLayout = QtWidgets.QVBoxLayout(SettingsDialogForm)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtWidgets.QTabWidget(SettingsDialogForm)
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.tab)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.downloadsLocation = QtWidgets.QLineEdit(self.tab)
        self.downloadsLocation.setObjectName("downloadsLocation")
        self.gridLayout_4.addWidget(self.downloadsLocation, 4, 1, 1, 2)
        spacerItem = QtWidgets.QSpacerItem(280, 18, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem, 1, 2, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.tab)
        self.label_3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_3.setObjectName("label_3")
        self.gridLayout_4.addWidget(self.label_3, 0, 0, 1, 1)
        self.home = QtWidgets.QLineEdit(self.tab)
        self.home.setMinimumSize(QtCore.QSize(400, 0))
        self.home.setObjectName("home")
        self.gridLayout_4.addWidget(self.home, 0, 1, 1, 2)
        self.goToHomePageOnOpen = QtWidgets.QCheckBox(self.tab)
        self.goToHomePageOnOpen.setObjectName("goToHomePageOnOpen")
        self.gridLayout_4.addWidget(self.goToHomePageOnOpen, 1, 1, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(391, 262, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem1, 6, 1, 1, 2)
        self.label_7 = QtWidgets.QLabel(self.tab)
        self.label_7.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_7.setObjectName("label_7")
        self.gridLayout_4.addWidget(self.label_7, 4, 0, 1, 1)
        self.changeDownloadsPathButton = QtWidgets.QPushButton(self.tab)
        self.changeDownloadsPathButton.setMaximumSize(QtCore.QSize(200, 16777215))
        self.changeDownloadsPathButton.setObjectName("changeDownloadsPathButton")
        self.gridLayout_4.addWidget(self.changeDownloadsPathButton, 5, 1, 1, 1)
        self.tabWidget.addTab(self.tab, "")
        self.tab_6 = QtWidgets.QWidget()
        self.tab_6.setObjectName("tab_6")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.tab_6)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setSizeConstraint(QtWidgets.QLayout.SetNoConstraint)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.groupBoxDaemon = QtWidgets.QGroupBox(self.tab_6)
        self.groupBoxDaemon.setMinimumSize(QtCore.QSize(0, 50))
        self.groupBoxDaemon.setCheckable(True)
        self.groupBoxDaemon.setObjectName("groupBoxDaemon")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.groupBoxDaemon)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.label_17 = QtWidgets.QLabel(self.groupBoxDaemon)
        self.label_17.setObjectName("label_17")
        self.gridLayout_5.addWidget(self.label_17, 2, 0, 1, 1)
        self.ipfsdApiPort = QtWidgets.QSpinBox(self.groupBoxDaemon)
        self.ipfsdApiPort.setMaximumSize(QtCore.QSize(100, 16777215))
        self.ipfsdApiPort.setMinimum(1024)
        self.ipfsdApiPort.setMaximum(32768)
        self.ipfsdApiPort.setObjectName("ipfsdApiPort")
        self.gridLayout_5.addWidget(self.ipfsdApiPort, 1, 1, 1, 1)
        self.ipfsdGwPort = QtWidgets.QSpinBox(self.groupBoxDaemon)
        self.ipfsdGwPort.setMaximumSize(QtCore.QSize(100, 16777215))
        self.ipfsdGwPort.setMinimum(1024)
        self.ipfsdGwPort.setMaximum(32768)
        self.ipfsdGwPort.setObjectName("ipfsdGwPort")
        self.gridLayout_5.addWidget(self.ipfsdGwPort, 2, 1, 1, 1)
        self.label_18 = QtWidgets.QLabel(self.groupBoxDaemon)
        self.label_18.setObjectName("label_18")
        self.gridLayout_5.addWidget(self.label_18, 0, 2, 1, 1)
        self.swarmMinConns = QtWidgets.QSpinBox(self.groupBoxDaemon)
        self.swarmMinConns.setMinimum(1)
        self.swarmMinConns.setMaximum(100)
        self.swarmMinConns.setObjectName("swarmMinConns")
        self.gridLayout_5.addWidget(self.swarmMinConns, 0, 3, 1, 1)
        self.label_19 = QtWidgets.QLabel(self.groupBoxDaemon)
        self.label_19.setObjectName("label_19")
        self.gridLayout_5.addWidget(self.label_19, 1, 2, 1, 1)
        self.ipfsdSwarmPort = QtWidgets.QSpinBox(self.groupBoxDaemon)
        self.ipfsdSwarmPort.setMaximumSize(QtCore.QSize(100, 16777215))
        self.ipfsdSwarmPort.setMinimum(1024)
        self.ipfsdSwarmPort.setMaximum(32768)
        self.ipfsdSwarmPort.setObjectName("ipfsdSwarmPort")
        self.gridLayout_5.addWidget(self.ipfsdSwarmPort, 0, 1, 1, 1)
        self.label_15 = QtWidgets.QLabel(self.groupBoxDaemon)
        self.label_15.setObjectName("label_15")
        self.gridLayout_5.addWidget(self.label_15, 1, 0, 1, 1)
        self.swarmMaxConns = QtWidgets.QSpinBox(self.groupBoxDaemon)
        self.swarmMaxConns.setMinimum(10)
        self.swarmMaxConns.setMaximum(300)
        self.swarmMaxConns.setObjectName("swarmMaxConns")
        self.gridLayout_5.addWidget(self.swarmMaxConns, 1, 3, 1, 1)
        self.label_14 = QtWidgets.QLabel(self.groupBoxDaemon)
        self.label_14.setObjectName("label_14")
        self.gridLayout_5.addWidget(self.label_14, 0, 0, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.groupBoxDaemon)
        self.label_5.setObjectName("label_5")
        self.gridLayout_5.addWidget(self.label_5, 2, 2, 1, 1)
        self.storageMax = QtWidgets.QSpinBox(self.groupBoxDaemon)
        self.storageMax.setMinimum(5)
        self.storageMax.setMaximum(2048)
        self.storageMax.setObjectName("storageMax")
        self.gridLayout_5.addWidget(self.storageMax, 2, 3, 1, 1)
        self.verticalLayout_5.addWidget(self.groupBoxDaemon)
        self.groupBoxIpfsConn = QtWidgets.QGroupBox(self.tab_6)
        self.groupBoxIpfsConn.setEnabled(True)
        self.groupBoxIpfsConn.setMinimumSize(QtCore.QSize(0, 50))
        self.groupBoxIpfsConn.setFlat(False)
        self.groupBoxIpfsConn.setCheckable(False)
        self.groupBoxIpfsConn.setChecked(False)
        self.groupBoxIpfsConn.setObjectName("groupBoxIpfsConn")
        self.formLayout_3 = QtWidgets.QFormLayout(self.groupBoxIpfsConn)
        self.formLayout_3.setObjectName("formLayout_3")
        self.label_2 = QtWidgets.QLabel(self.groupBoxIpfsConn)
        self.label_2.setObjectName("label_2")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.label_16 = QtWidgets.QLabel(self.groupBoxIpfsConn)
        self.label_16.setObjectName("label_16")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_16)
        self.customIpfsApiPort = QtWidgets.QSpinBox(self.groupBoxIpfsConn)
        self.customIpfsApiPort.setMaximumSize(QtCore.QSize(100, 16777215))
        self.customIpfsApiPort.setMinimum(1024)
        self.customIpfsApiPort.setMaximum(32768)
        self.customIpfsApiPort.setObjectName("customIpfsApiPort")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.customIpfsApiPort)
        self.label = QtWidgets.QLabel(self.groupBoxIpfsConn)
        self.label.setObjectName("label")
        self.formLayout_3.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label)
        self.customIpfsGwPort = QtWidgets.QSpinBox(self.groupBoxIpfsConn)
        self.customIpfsGwPort.setMaximumSize(QtCore.QSize(100, 16777215))
        self.customIpfsGwPort.setMinimum(1024)
        self.customIpfsGwPort.setMaximum(32768)
        self.customIpfsGwPort.setObjectName("customIpfsGwPort")
        self.formLayout_3.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.customIpfsGwPort)
        self.customIpfsHost = QtWidgets.QLineEdit(self.groupBoxIpfsConn)
        self.customIpfsHost.setMaximumSize(QtCore.QSize(300, 16777215))
        self.customIpfsHost.setObjectName("customIpfsHost")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.customIpfsHost)
        self.verticalLayout_5.addWidget(self.groupBoxIpfsConn)
        self.verticalLayout_4.addLayout(self.verticalLayout_5)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_4 = QtWidgets.QLabel(self.tab_6)
        self.label_4.setMaximumSize(QtCore.QSize(721, 145))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_3.addWidget(self.label_4)
        self.verticalLayout_4.addLayout(self.horizontalLayout_3)
        self.tabWidget.addTab(self.tab_6, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.tab_2)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.groupBox_2 = QtWidgets.QGroupBox(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_2.sizePolicy().hasHeightForWidth())
        self.groupBox_2.setSizePolicy(sizePolicy)
        self.groupBox_2.setObjectName("groupBox_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.groupBox_2)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_6 = QtWidgets.QLabel(self.groupBox_2)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout.addWidget(self.label_6)
        self.language = QtWidgets.QComboBox(self.groupBox_2)
        self.language.setObjectName("language")
        self.horizontalLayout.addWidget(self.language)
        self.horizontalLayout_2.addLayout(self.horizontalLayout)
        self.gridLayout.addWidget(self.groupBox_2, 0, 0, 1, 1)
        self.groupBox = QtWidgets.QGroupBox(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setObjectName("groupBox")
        self.wrapFiles = QtWidgets.QCheckBox(self.groupBox)
        self.wrapFiles.setGeometry(QtCore.QRect(10, 30, 661, 21))
        self.wrapFiles.setObjectName("wrapFiles")
        self.wrapDirectories = QtWidgets.QCheckBox(self.groupBox)
        self.wrapDirectories.setGeometry(QtCore.QRect(10, 60, 661, 21))
        self.wrapDirectories.setObjectName("wrapDirectories")
        self.hideHashes = QtWidgets.QCheckBox(self.groupBox)
        self.hideHashes.setGeometry(QtCore.QRect(10, 90, 661, 21))
        self.hideHashes.setObjectName("hideHashes")
        self.gridLayout.addWidget(self.groupBox, 1, 0, 1, 1)
        self.gridLayout_2.addLayout(self.gridLayout, 0, 0, 1, 1)
        self.groupBox_3 = QtWidgets.QGroupBox(self.tab_2)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.gridLayout_6.addLayout(self.gridLayout_3, 0, 0, 1, 1)
        self.jsIpfsApi = QtWidgets.QCheckBox(self.groupBox_3)
        self.jsIpfsApi.setObjectName("jsIpfsApi")
        self.gridLayout_6.addWidget(self.jsIpfsApi, 1, 0, 1, 1)
        self.gridLayout_2.addWidget(self.groupBox_3, 1, 0, 1, 1)
        self.tabWidget.addTab(self.tab_2, "")
        self.verticalLayout.addWidget(self.tabWidget)
        self.buttonBox = QtWidgets.QDialogButtonBox(SettingsDialogForm)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(SettingsDialogForm)
        self.tabWidget.setCurrentIndex(0)
        self.buttonBox.accepted.connect(SettingsDialogForm.accept)
        self.buttonBox.rejected.connect(SettingsDialogForm.reject)
        QtCore.QMetaObject.connectSlotsByName(SettingsDialogForm)

    def retranslateUi(self, SettingsDialogForm):
        _translate = QtCore.QCoreApplication.translate
        SettingsDialogForm.setWindowTitle(_translate("SettingsDialogForm", "Settings"))
        self.label_3.setText(_translate("SettingsDialogForm", "Home:"))
        self.goToHomePageOnOpen.setText(_translate("SettingsDialogForm", "Go to home page when opening a new browser tab"))
        self.label_7.setText(_translate("SettingsDialogForm", "Save downloads to:"))
        self.changeDownloadsPathButton.setText(_translate("SettingsDialogForm", "Change directory"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("SettingsDialogForm", "General"))
        self.groupBoxDaemon.setTitle(_translate("SettingsDialogForm", "Spawn IPFS daemon and use as transport"))
        self.label_17.setText(_translate("SettingsDialogForm", "HTTP gateway port"))
        self.label_18.setText(_translate("SettingsDialogForm", "Minimum swarm connections"))
        self.label_19.setText(_translate("SettingsDialogForm", "Maximum swarm connections"))
        self.label_15.setText(_translate("SettingsDialogForm", "API port"))
        self.label_14.setText(_translate("SettingsDialogForm", "Swarm port"))
        self.label_5.setText(_translate("SettingsDialogForm", "Maximum storage (Gb)"))
        self.groupBoxIpfsConn.setTitle(_translate("SettingsDialogForm", "Use a custom IPFS daemon"))
        self.label_2.setText(_translate("SettingsDialogForm", "Host"))
        self.label_16.setText(_translate("SettingsDialogForm", "API port"))
        self.label.setText(_translate("SettingsDialogForm", "HTTP gateway port"))
        self.label_4.setText(_translate("SettingsDialogForm", "Restart the application after changing the IPFS connection settings."))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_6), _translate("SettingsDialogForm", "IPFS"))
        self.groupBox_2.setTitle(_translate("SettingsDialogForm", "Language"))
        self.label_6.setText(_translate("SettingsDialogForm", "Language"))
        self.groupBox.setTitle(_translate("SettingsDialogForm", "File manager"))
        self.wrapFiles.setText(_translate("SettingsDialogForm", "Wrap files with a directory (preserves filenames)"))
        self.wrapDirectories.setText(_translate("SettingsDialogForm", "Wrap directories with a directory"))
        self.hideHashes.setText(_translate("SettingsDialogForm", "Hide IPFS hashes in tree views"))
        self.groupBox_3.setTitle(_translate("SettingsDialogForm", "Browser"))
        self.jsIpfsApi.setText(_translate("SettingsDialogForm", "Enable Javascript IPFS API"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), _translate("SettingsDialogForm", "UI"))

