# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/files.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_FileManagerForm(object):
    def setupUi(self, FileManagerForm):
        FileManagerForm.setObjectName("FileManagerForm")
        FileManagerForm.resize(618, 349)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(FileManagerForm)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSizeConstraint(QtWidgets.QLayout.SetFixedSize)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.pathSelector = QtWidgets.QComboBox(FileManagerForm)
        self.pathSelector.setObjectName("pathSelector")
        self.horizontalLayout_2.addWidget(self.pathSelector)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.addFileButton = QtWidgets.QToolButton(FileManagerForm)
        self.addFileButton.setMaximumSize(QtCore.QSize(32, 32))
        self.addFileButton.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/share/icons/add-file.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.addFileButton.setIcon(icon)
        self.addFileButton.setIconSize(QtCore.QSize(24, 24))
        self.addFileButton.setObjectName("addFileButton")
        self.horizontalLayout_2.addWidget(self.addFileButton)
        self.addDirectoryButton = QtWidgets.QToolButton(FileManagerForm)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/share/icons/add-folder.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.addDirectoryButton.setIcon(icon1)
        self.addDirectoryButton.setIconSize(QtCore.QSize(24, 24))
        self.addDirectoryButton.setObjectName("addDirectoryButton")
        self.horizontalLayout_2.addWidget(self.addDirectoryButton)
        self.refreshButton = QtWidgets.QToolButton(FileManagerForm)
        self.refreshButton.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/share/icons/refresh.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.refreshButton.setIcon(icon2)
        self.refreshButton.setIconSize(QtCore.QSize(24, 24))
        self.refreshButton.setObjectName("refreshButton")
        self.horizontalLayout_2.addWidget(self.refreshButton)
        self.horizontalLayout.addLayout(self.horizontalLayout_2)
        spacerItem1 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.fileManagerButton = QtWidgets.QToolButton(FileManagerForm)
        self.fileManagerButton.setText("")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/share/icons/file-manager.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.fileManagerButton.setIcon(icon3)
        self.fileManagerButton.setIconSize(QtCore.QSize(24, 24))
        self.fileManagerButton.setObjectName("fileManagerButton")
        self.horizontalLayout.addWidget(self.fileManagerButton)
        self.statusLabel = QtWidgets.QLabel(FileManagerForm)
        self.statusLabel.setMinimumSize(QtCore.QSize(200, 0))
        self.statusLabel.setText("")
        self.statusLabel.setObjectName("statusLabel")
        self.horizontalLayout.addWidget(self.statusLabel)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label = QtWidgets.QLabel(FileManagerForm)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setMaximumSize(QtCore.QSize(100, 16777215))
        self.label.setObjectName("label")
        self.horizontalLayout_3.addWidget(self.label)
        self.searchFiles = QtWidgets.QLineEdit(FileManagerForm)
        self.searchFiles.setMaximumSize(QtCore.QSize(500, 16777215))
        self.searchFiles.setObjectName("searchFiles")
        self.horizontalLayout_3.addWidget(self.searchFiles)
        self.horizontalLayout.addLayout(self.horizontalLayout_3)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.hLayoutFilesView = QtWidgets.QHBoxLayout()
        self.hLayoutFilesView.setObjectName("hLayoutFilesView")
        self.treeFiles = QtWidgets.QTreeView(FileManagerForm)
        self.treeFiles.setObjectName("treeFiles")
        self.treeFiles.header().setCascadingSectionResizes(True)
        self.treeFiles.header().setDefaultSectionSize(140)
        self.treeFiles.header().setStretchLastSection(True)
        self.hLayoutFilesView.addWidget(self.treeFiles)
        self.hLayoutBrowser = QtWidgets.QHBoxLayout()
        self.hLayoutBrowser.setObjectName("hLayoutBrowser")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.fileManagerSwitch = QtWidgets.QPushButton(FileManagerForm)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.fileManagerSwitch.sizePolicy().hasHeightForWidth())
        self.fileManagerSwitch.setSizePolicy(sizePolicy)
        self.fileManagerSwitch.setMaximumSize(QtCore.QSize(128, 16777215))
        self.fileManagerSwitch.setText("")
        self.fileManagerSwitch.setIcon(icon3)
        self.fileManagerSwitch.setObjectName("fileManagerSwitch")
        self.verticalLayout_4.addWidget(self.fileManagerSwitch)
        self.hLayoutBrowser.addLayout(self.verticalLayout_4)
        self.hLayoutFilesView.addLayout(self.hLayoutBrowser)
        self.verticalLayout.addLayout(self.hLayoutFilesView)
        self.verticalLayout_2.addLayout(self.verticalLayout)

        self.retranslateUi(FileManagerForm)
        QtCore.QMetaObject.connectSlotsByName(FileManagerForm)

    def retranslateUi(self, FileManagerForm):
        _translate = QtCore.QCoreApplication.translate
        FileManagerForm.setWindowTitle(_translate("FileManagerForm", "Form"))
        self.addFileButton.setToolTip(_translate("FileManagerForm", "<html><head/><body><p>Add Files</p></body></html>"))
        self.addDirectoryButton.setToolTip(_translate("FileManagerForm", "<html><head/><body><p>Add directory</p></body></html>"))
        self.addDirectoryButton.setText(_translate("FileManagerForm", "Add directory"))
        self.refreshButton.setToolTip(_translate("FileManagerForm", "<html><head/><body><p>Refresh</p></body></html>"))
        self.fileManagerButton.setToolTip(_translate("FileManagerForm", "<html><head/><body><p>Open local file manager</p></body></html>"))
        self.label.setText(_translate("FileManagerForm", "Search"))
        self.fileManagerSwitch.setToolTip(_translate("FileManagerForm", "<html><head/><body><p>Open local file manager</p></body></html>"))

from . import galacteek_rc
