# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/ipfsinfos.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_IPFSInfosDialog(object):
    def setupUi(self, IPFSInfosDialog):
        IPFSInfosDialog.setObjectName("IPFSInfosDialog")
        IPFSInfosDialog.resize(428, 368)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(IPFSInfosDialog)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtWidgets.QTabWidget(IPFSInfosDialog)
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.tab)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.gridLayout.setObjectName("gridLayout")
        self.repoSize = QtWidgets.QLabel(self.tab)
        self.repoSize.setText("")
        self.repoSize.setObjectName("repoSize")
        self.gridLayout.addWidget(self.repoSize, 1, 1, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.tab)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 2, 0, 1, 1)
        self.label = QtWidgets.QLabel(self.tab)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.repoObjCount = QtWidgets.QLabel(self.tab)
        self.repoObjCount.setText("")
        self.repoObjCount.setObjectName("repoObjCount")
        self.gridLayout.addWidget(self.repoObjCount, 0, 1, 1, 1)
        self.repoMaxStorage = QtWidgets.QLabel(self.tab)
        self.repoMaxStorage.setText("")
        self.repoMaxStorage.setObjectName("repoMaxStorage")
        self.gridLayout.addWidget(self.repoMaxStorage, 2, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.tab)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 1, 0, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.tab)
        self.label_7.setObjectName("label_7")
        self.gridLayout.addWidget(self.label_7, 3, 0, 1, 1)
        self.repoVersion = QtWidgets.QLabel(self.tab)
        self.repoVersion.setText("")
        self.repoVersion.setObjectName("repoVersion")
        self.gridLayout.addWidget(self.repoVersion, 3, 1, 1, 1)
        self.gridLayout_2.addLayout(self.gridLayout, 0, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem, 1, 0, 1, 1)
        self.tabWidget.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.tab_2)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.label_6 = QtWidgets.QLabel(self.tab_2)
        self.label_6.setObjectName("label_6")
        self.gridLayout_3.addWidget(self.label_6, 1, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.tab_2)
        self.label_2.setObjectName("label_2")
        self.gridLayout_3.addWidget(self.label_2, 0, 0, 1, 1)
        self.agentVersion = QtWidgets.QLabel(self.tab_2)
        self.agentVersion.setText("")
        self.agentVersion.setObjectName("agentVersion")
        self.gridLayout_3.addWidget(self.agentVersion, 1, 1, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem1, 3, 0, 1, 1)
        self.nodeId = QtWidgets.QLabel(self.tab_2)
        self.nodeId.setText("")
        self.nodeId.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.nodeId.setObjectName("nodeId")
        self.gridLayout_3.addWidget(self.nodeId, 0, 1, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.tab_2)
        self.label_4.setObjectName("label_4")
        self.gridLayout_3.addWidget(self.label_4, 2, 0, 1, 1)
        self.protocolVersion = QtWidgets.QLabel(self.tab_2)
        self.protocolVersion.setText("")
        self.protocolVersion.setObjectName("protocolVersion")
        self.gridLayout_3.addWidget(self.protocolVersion, 2, 1, 1, 1)
        self.verticalLayout_3.addLayout(self.gridLayout_3)
        self.tabWidget.addTab(self.tab_2, "")
        self.verticalLayout.addWidget(self.tabWidget)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.okButton = QtWidgets.QPushButton(IPFSInfosDialog)
        self.okButton.setObjectName("okButton")
        self.verticalLayout_2.addWidget(self.okButton)

        self.retranslateUi(IPFSInfosDialog)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(IPFSInfosDialog)

    def retranslateUi(self, IPFSInfosDialog):
        _translate = QtCore.QCoreApplication.translate
        IPFSInfosDialog.setWindowTitle(_translate("IPFSInfosDialog", "Dialog"))
        self.label_5.setText(_translate("IPFSInfosDialog", "Maximum storage"))
        self.label.setText(_translate("IPFSInfosDialog", "Objects count"))
        self.label_3.setText(_translate("IPFSInfosDialog", "Repository size"))
        self.label_7.setText(_translate("IPFSInfosDialog", "Version"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("IPFSInfosDialog", "Repository"))
        self.label_6.setText(_translate("IPFSInfosDialog", "Agent version"))
        self.label_2.setText(_translate("IPFSInfosDialog", "ID"))
        self.label_4.setText(_translate("IPFSInfosDialog", "Protocol version"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), _translate("IPFSInfosDialog", "Node"))
        self.okButton.setText(_translate("IPFSInfosDialog", "OK"))

